@@ -13,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import warnings
+
 from neutron_lib.api import converters
 from neutron_lib.api import extensions as api_extensions
 from neutron_lib.api import faults
@@ -20,6 +22,7 @@ from neutron_lib.db import constants as const
 from neutron_lib import exceptions
 from neutron_lib.plugins import directory
 from oslo_config import cfg
+from oslo_log import log as logging
 from oslo_utils import importutils
 import webob
 
@@ -27,24 +30,24 @@ from neutron._i18n import _
 from neutron.api import extensions
 from neutron.api.v2 import base
 from neutron.api.v2 import resource
+from neutron.api import wsgi
 from neutron import policy
 from neutron import quota
 from neutron.quota import resource_registry
-from neutron import wsgi
 
 
 DEFAULT_QUOTAS_ACTION = 'default'
 RESOURCE_NAME = 'quota'
 RESOURCE_COLLECTION = RESOURCE_NAME + "s"
 QUOTAS = quota.QUOTAS
-DB_QUOTA_DRIVER = 'neutron.db.quota.driver.DbQuotaDriver'
+DB_QUOTA_DRIVER = cfg.CONF.QUOTAS.quota_driver
 EXTENDED_ATTRIBUTES_2_0 = {
     RESOURCE_COLLECTION: {}
 }
+LOG = logging.getLogger(__name__)
 
 
 def validate_policy(context, policy_name):
-    policy.init()
     policy.enforce(context,
                    policy_name,
                    target={'project_id': context.project_id},
@@ -56,9 +59,7 @@ class QuotaSetsController(wsgi.Controller):
     def __init__(self, plugin):
         self._resource_name = RESOURCE_NAME
         self._plugin = plugin
-        self._driver = importutils.import_class(
-            cfg.CONF.QUOTAS.quota_driver
-        )
+        self._driver = importutils.import_class(cfg.CONF.QUOTAS.quota_driver)()
         self._update_extended_attributes = True
 
     def _update_attributes(self):
@@ -72,20 +73,21 @@ class QuotaSetsController(wsgi.Controller):
                 'is_visible': True}
         self._update_extended_attributes = False
 
-    def _get_quotas(self, request, tenant_id):
-        return self._driver.get_tenant_quotas(
+    def _get_quotas(self, request, project_id):
+        return self._driver.get_project_quotas(
             request.context,
             resource_registry.get_all_resources(),
-            tenant_id)
+            project_id)
 
     def default(self, request, id):
         context = request.context
-        if id != context.tenant_id:
+        if id != context.project_id:
             validate_policy(context, "get_quota")
-        return {self._resource_name: self._driver.get_default_quotas(
-                   context=context,
-                   resources=resource_registry.get_all_resources(),
-                   tenant_id=id)}
+        return {self._resource_name:
+                self._driver.get_default_quotas(
+                    context=context,
+                    resources=resource_registry.get_all_resources(),
+                    project_id=id)}
 
     def create(self, request, body=None):
         msg = _('POST requests are not supported on this resource.')
@@ -99,28 +101,75 @@ class QuotaSetsController(wsgi.Controller):
                     context, resource_registry.get_all_resources())}
 
     def tenant(self, request):
-        """Retrieve the tenant info in context."""
+        """Retrieve the project info in context."""
+        warnings.warn(
+            '"tenant" Quota API method is deprecated, use "project" instead')
+        return self._project(request, 'tenant')
+
+    def project(self, request):
+        """Retrieve the project info in context."""
+        return self._project(request, 'project')
+
+    @staticmethod
+    def _project(request, key):
+        """Retrieve the project info in context."""
         context = request.context
-        if not context.tenant_id:
+        if not context.project_id:
             raise exceptions.QuotaMissingTenant()
-        return {'tenant': {'tenant_id': context.tenant_id}}
+        return {key: {key + '_id': context.project_id}}
 
     def show(self, request, id):
-        if id != request.context.tenant_id:
+        if id != request.context.project_id:
             validate_policy(request.context, "get_quota")
         return {self._resource_name: self._get_quotas(request, id)}
 
     def delete(self, request, id):
         validate_policy(request.context, "delete_quota")
-        self._driver.delete_tenant_quota(request.context, id)
+        self._driver.delete_project_quota(request.context, id)
 
     def update(self, request, id, body=None):
         validate_policy(request.context, "update_quota")
+        force = body[self._resource_name].pop('force', None)
+        check_limit = body[self._resource_name].pop('check_limit', None)
+        # NOTE(ralonsoh): these warning messages will be removed once
+        # LP#1953170 is completed and Neutron quota engine accepts "--force" or
+        # nothing (by default, Neutron quota engine will check the resource
+        # usage before setting the quota limit).
+        if force is None and check_limit is None:
+            warnings.warn('Neutron quota engine will require "--force" '
+                          'parameter to set a quota limit without checking '
+                          'the resource usage.')
+        elif check_limit:
+            warnings.warn('"--check-limit" parameter will not be needed in '
+                          'Z+. By default, Neutron quota engine will check '
+                          'the resource usage before setting a new quota '
+                          'limit. Use "--force" to skip this check.')
+
         if self._update_extended_attributes:
             self._update_attributes()
-        body = base.Controller.prepare_request_body(
-            request.context, body, False, self._resource_name,
-            EXTENDED_ATTRIBUTES_2_0[RESOURCE_COLLECTION])
+        try:
+            body = base.Controller.prepare_request_body(
+                request.context, body, False, self._resource_name,
+                EXTENDED_ATTRIBUTES_2_0[RESOURCE_COLLECTION])
+        except Exception as e:
+            LOG.warning(
+                "An exception happened while processing the request "
+                "body. The exception message is [%s].", e)
+            raise e
+
+        if check_limit:
+            resources = resource_registry.get_all_resources()
+            for resource_name, limit in body[self._resource_name].items():
+                resource_usage = self._driver.get_resource_usage(
+                    request.context, id, resources, resource_name)
+                if resource_usage > limit:
+                    msg = ('Quota limit %(limit)s for %(resource)s must be '
+                           'greater than or equal to already used '
+                           '%(resource_usage)s' %
+                           {'limit': limit, 'resource': resource_name,
+                            'resource_usage': resource_usage})
+                    raise webob.exc.HTTPBadRequest(msg)
+
         for key, value in body[self._resource_name].items():
             self._driver.update_quota_limit(request.context, id, key, value)
         return {self._resource_name: self._get_quotas(request, id)}
@@ -144,7 +193,7 @@ class Quotasv2(api_extensions.ExtensionDescriptor):
     def get_description(cls):
         description = 'Expose functions for quotas management'
         if cfg.CONF.QUOTAS.quota_driver == DB_QUOTA_DRIVER:
-            description += ' per tenant'
+            description += ' per project'
         return description
 
     @classmethod
@@ -161,7 +210,8 @@ class Quotasv2(api_extensions.ExtensionDescriptor):
             Quotasv2.get_alias(),
             controller,
             member_actions={DEFAULT_QUOTAS_ACTION: 'GET'},
-            collection_actions={'tenant': 'GET'})]
+            collection_actions={'tenant': 'GET',
+                                'project': 'GET'})]
 
     def get_extended_resources(self, version):
         if version == "2.0":
