@@ -127,6 +127,7 @@ class RouterExtraAttributes(base.NeutronDbObject):
                           l3_attrs.RouterExtraAttributes.router_id).
                      join(l3.Router).
                      group_by(binding_model.router_id).subquery())
+        # pylint: disable=assignment-from-no-return
         count = func.coalesce(sub_query.c.count, 0)
         query = (context.session.query(l3.Router, count).
                  outerjoin(sub_query).join(l3_attrs.RouterExtraAttributes).
@@ -136,6 +137,12 @@ class RouterExtraAttributes(base.NeutronDbObject):
 
         return list(query)
 
+    @classmethod
+    @db_api.CONTEXT_WRITER
+    def update_distributed_flag(cls, context, distributed):
+        query = context.session.query(cls.db_model)
+        query.update({'distributed': distributed})
+
     def obj_make_compatible(self, primitive, target_version):
         _target_version = versionutils.convert_version_to_tuple(target_version)
         if _target_version < (1, 1):
