@@ -34,7 +34,7 @@ from neutron.agent import l3_agent as l3_agent_main
 from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import keepalived
-from neutron.agent.metadata import driver as metadata_driver
+from neutron.agent.metadata import driver_base
 from neutron.common import utils as common_utils
 from neutron.conf.agent import common as agent_config
 from neutron.conf.agent.l3 import config as l3_config
@@ -78,11 +78,11 @@ vrrp_instance VR_1 {
         %(int_port_ipv6)s dev %(internal_device_name)s scope link no_track
     }
     virtual_routes {
-        0.0.0.0/0 via %(default_gateway_ip)s dev %(ex_device_name)s no_track
-        8.8.8.0/24 via 19.4.4.4 no_track
-        %(extra_subnet_cidr)s dev %(ex_device_name)s scope link no_track
+        0.0.0.0/0 via %(default_gateway_ip)s dev %(ex_device_name)s no_track protocol static
+        8.8.8.0/24 via 19.4.4.4 no_track protocol static
+        %(extra_subnet_cidr)s dev %(ex_device_name)s scope link no_track protocol static
     }
-}"""
+}"""  # noqa: E501 # pylint: disable=line-too-long
 
 
 def get_ovs_bridge(br_name):
@@ -137,6 +137,11 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         conf.set_override('host', host)
         conf.set_override('agent_mode', agent_mode)
 
+        # NOTE(slaweq): iptables_manager module checks this option directly in
+        # the cfg.CONF, not in agent.conf parameter so it has to be override
+        # directly in the cfg.CONF module too
+        cfg.CONF.set_override('debug_iptables_rules', True, group='AGENT')
+
         return conf
 
     def _get_agent_ovs_integration_bridge(self, agent):
@@ -237,8 +242,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             (fip['id'], constants.FLOATINGIP_STATUS_ACTIVE) for fip in
             router.router[constants.FLOATINGIP_KEY]])
         call = [args[0] for args in rpc.call_args_list][0]
-        actual_fips = set(
-            [(fip_id, status) for fip_id, status in call[2].items()])
+        actual_fips = set(list(call[2].items()))
         self.assertEqual(expected_fips, actual_fips)
 
     def _gateway_check(self, gateway_ip, external_device):
@@ -406,6 +410,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
                 timeout=15)
         return return_copy
 
+    @test_base.unstable_test("bug 1961740")
     def manage_router(self, agent, router):
         self.addCleanup(agent._safe_router_removed, router['id'])
         with mock.patch.object(dvr_local_router.DvrLocalRouter,
@@ -444,7 +449,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             conf,
             router.router_id,
             router.ns_name,
-            service=metadata_driver.HAPROXY_SERVICE)
+            service=driver_base.HAPROXY_SERVICE)
 
     def _metadata_proxy_exists(self, conf, router):
         pm = self._metadata_proxy(conf, router)
