@@ -14,6 +14,7 @@
 import datetime
 import functools
 import subprocess
+import time
 from unittest import mock
 
 import fixtures as og_fixtures
@@ -32,6 +33,7 @@ from sqlalchemy.dialects.mysql import dialect as mysql_dialect
 import tenacity
 
 from neutron.common.ovn import constants as ovn_const
+from neutron.common.ovn import utils as ovn_utils
 from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db as db_hash_ring
@@ -83,6 +85,24 @@ class DistributedLockTestEvent(event.WaitEvent):
         self.event.set()
 
 
+class WaitForLogicalSwitchPortUpdateEvent(event.WaitEvent):
+    event_name = 'WaitForDataPathBindingCreateEvent'
+
+    def __init__(self):
+        table = 'Logical_Switch_Port'
+        events = (self.ROW_UPDATE,)
+        super().__init__(events, table, None, timeout=15)
+
+
+class WaitForLogicalRouterPortCreateEvent(event.WaitEvent):
+    event_name = 'WaitForLogicalRouterPortCreateEvent'
+
+    def __init__(self):
+        table = 'Logical_Router_Port'
+        events = (self.ROW_CREATE,)
+        super().__init__(events, table, None, timeout=15)
+
+
 class GlobalTestEvent(DistributedLockTestEvent):
     GLOBAL = True
 
@@ -538,6 +558,52 @@ class TestNBDbMonitor(testlib_api.MySQLTestCaseMixin,
                 timeout=10)
             self._check_port_and_port_binding_revision_number(vip['id'])
 
+    def _create_router(self):
+        net_args = {external_net.EXTERNAL: True,
+                    provider_net.NETWORK_TYPE: 'geneve'}
+        net = self._make_network(self.fmt, 'e1', True, as_admin=True,
+                                 arg_list=tuple(net_args.keys()), **net_args)
+        res = self._create_subnet(self.fmt, net['network']['id'],
+                                  '120.0.0.0/24')
+        subnet = self.deserialize(self.fmt, res)
+        external_gateway_info = {
+            'enable_snat': True, 'network_id': net['network']['id'],
+            'external_fixed_ips': [{'ip_address': '120.0.0.2',
+                                    'subnet_id': subnet['subnet']['id']}]}
+        router = self.l3_plugin.create_router(
+            self.context,
+            {'router': {'name': uuidutils.generate_uuid(),
+                        'admin_state_up': True, 'tenant_id': self._tenant_id,
+                        'external_gateway_info': external_gateway_info}})
+        return router
+
+    def test_ha_chassis_group_router_event(self):
+        def _check_high_prio_chassis(num_chassis):
+            lr = self.nb_api.lookup('Logical_Router', ovn_r_name)
+            hp_chassis_lr = lr.options['chassis']
+            hcg = self.nb_api.lookup('HA_Chassis_Group', ovn_r_name)
+            self.assertEqual(num_chassis, len(hcg.ha_chassis))
+            hp_chassis_hcg = max(hcg.ha_chassis, key=lambda hc: hc.priority)
+            self.assertEqual(hp_chassis_lr, hp_chassis_hcg.chassis_name)
+
+        chassis_list = []
+        num_chassis = 5
+        for idx in range(num_chassis):
+            chassis_list.append(
+                self.add_fake_chassis('host-%s' % str(idx), azs=[],
+                                      enable_chassis_as_gw=True))
+
+        router = self._create_router()
+        ovn_r_name = ovn_utils.ovn_name(router['id'])
+        _check_high_prio_chassis(len(chassis_list))
+
+        lr = self.nb_api.lookup('Logical_Router', ovn_r_name)
+        row_event = test_events.WaitForLogicalRouterUpdate()
+        self.mech_driver.nb_ovn.idl.notify_handler.watch_event(row_event)
+        self.del_fake_chassis(lr.options['chassis'])
+        self.assertTrue(row_event.wait())
+        _check_high_prio_chassis(num_chassis - 1)
+
 
 class TestNBDbMonitorOverTcp(TestNBDbMonitor):
     def get_ovsdb_server_protocol(self):
@@ -573,7 +639,7 @@ class TestSBDbMonitor(base.TestOVNFunctionalBase, test_l3.L3NatTestCaseMixin):
         def check_ext_ids():
             pb = self._find_port_binding(ovn_const.OVN_CHASSIS_REDIRECT)
             _, lrp = list(
-                self.nb_api.tables['Logical_Router_Port'].rows.data.items())[0]
+                self.nb_api.tables['Logical_Router_Port'].rows.items())[0]
             if pb.external_ids == {}:
                 # The current version of OVN installed in FT CI could not have
                 # [1]. In this case, the Port_Binding.external_ids value is an
@@ -607,7 +673,7 @@ class TestSBDbMonitor(base.TestOVNFunctionalBase, test_l3.L3NatTestCaseMixin):
         except n_utils.WaitTimeout:
             pb = self._find_port_binding(ovn_const.OVN_CHASSIS_REDIRECT)
             _, lrp = list(
-                self.nb_api.tables['Logical_Router_Port'].rows.data.items())[0]
+                self.nb_api.tables['Logical_Router_Port'].rows.items())[0]
             self.fail('pb.ext_ids: %s  --  lrp.ext_ids: %s' %
                       (pb.external_ids, lrp.external_ids))
 
@@ -845,3 +911,57 @@ class TestPortBindingChassisEvent(base.TestOVNFunctionalBase,
         self.sb_api.lsp_bind(self.port['id'], self.chassis,
                              may_exist=True).execute(check_error=True)
         self._check_pb_type('')
+
+
+class TestLogicalSwitchPortUpdateLogicalRouterPortEvent(
+        base.TestOVNFunctionalBase,
+        test_l3.L3NatTestCaseMixin):
+
+    def setUp(self, **kwargs):
+        super().setUp(**kwargs)
+        self.chassis = self.add_fake_chassis('ovs-host1')
+        self.l3_plugin = directory.get_plugin(plugin_constants.L3)
+        self.net = self._make_network(
+            self.fmt, 'ext_net', True, as_admin=True, **kwargs)
+        self.subnet = self._make_subnet(self.fmt, self.net, '20.0.10.1',
+                                        '20.0.10.0/24')
+        self.ext_api = test_extensions.setup_extensions_middleware(
+            test_l3.L3TestExtensionManager())
+
+    def _set_logical_port_events_add_subnet_to_router(self):
+        lsp_event = WaitForLogicalSwitchPortUpdateEvent()
+        lrp_event = WaitForLogicalRouterPortCreateEvent()
+        self.mech_driver.nb_ovn.idl.notify_handler.watch_events(
+            (lsp_event, lrp_event))
+        router = self._make_router(self.fmt, self._tenant_id)
+        self._router_interface_action('add', router['router']['id'],
+                                      self.subnet['subnet']['id'], None)
+        self.assertTrue(lsp_event.wait())
+        self.assertTrue(lrp_event.wait())
+        # Wait for the
+        # ``LogicalSwitchPortUpdateLogicalRouterPortEvent.run`` call.
+        time.sleep(1)
+
+    def test_create_router_port(self):
+        with mock.patch.object(self.l3_plugin._ovn_client,
+                               'update_router_port') as mock_update_rp:
+            self._set_logical_port_events_add_subnet_to_router()
+            mock_update_rp.assert_called()
+
+    def test_create_router_port_port_deleted_concurrently(self):
+        with mock.patch.object(self.l3_plugin._ovn_client,
+                               'update_router_port') as mock_update_rp, \
+                mock.patch.object(self.plugin, 'get_ports', return_value=[]):
+            self._set_logical_port_events_add_subnet_to_router()
+            mock_update_rp.assert_not_called()
+
+    def test_create_non_router_port(self):
+        with mock.patch.object(self.l3_plugin._ovn_client,
+                               'update_router_port') as mock_update_rp:
+            row_event = WaitForLogicalSwitchPortUpdateEvent()
+            self.mech_driver.nb_ovn.idl.notify_handler.watch_event(row_event)
+            self._create_port(self.fmt, self.net['network']['id'])
+            # The LogicalSwitchPort event is called but not the router port
+            # update method because this is not a router port.
+            self.assertTrue(row_event.wait())
+            mock_update_rp.assert_not_called()
