@@ -23,9 +23,9 @@ import webtest
 
 from neutron._i18n import _
 from neutron.api.v2 import resource as wsgi_resource
+from neutron.api import wsgi
 from neutron.common import utils
 from neutron.tests import base
-from neutron import wsgi
 
 
 class RequestTestCase(base.BaseTestCase):
@@ -138,7 +138,7 @@ class ResourceTestCase(base.BaseTestCase):
         return wsgi.JSONDeserializer()
 
     def test_unmapped_neutron_error_with_json(self):
-        msg = u'\u7f51\u7edc'
+        msg = '\u7f51\u7edc'
 
         class TestException(n_exc.NeutronException):
             message = msg
@@ -181,7 +181,7 @@ class ResourceTestCase(base.BaseTestCase):
                       str(wsgi.JSONDeserializer().deserialize(res.body)))
 
     def test_mapped_neutron_error_with_json(self):
-        msg = u'\u7f51\u7edc'
+        msg = '\u7f51\u7edc'
 
         class TestException(n_exc.NeutronException):
             message = msg
