@@ -40,6 +40,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.dhcp_options_table = FakeOvsdbTable.create_one_ovsdb_table()
         self.nat_table = FakeOvsdbTable.create_one_ovsdb_table()
         self.port_group_table = FakeOvsdbTable.create_one_ovsdb_table()
+        self.gateway_chassis_table = FakeOvsdbTable.create_one_ovsdb_table()
         self._tables = {}
         self._tables['Logical_Switch'] = self.lswitch_table
         self._tables['Logical_Switch_Port'] = self.lsp_table
@@ -52,6 +53,7 @@ class FakeOvsdbNbOvnIdl(object):
         self._tables['DHCP_Options'] = self.dhcp_options_table
         self._tables['NAT'] = self.nat_table
         self._tables['Port_Group'] = self.port_group_table
+        self._tables['Gateway_Chassis'] = self.gateway_chassis_table
         self.transaction = mock.MagicMock()
         self.create_transaction = mock.MagicMock()
         self.ls_add = mock.Mock()
@@ -60,11 +62,11 @@ class FakeOvsdbNbOvnIdl(object):
         self.set_lswitch_port = mock.Mock()
         self.delete_lswitch_port = mock.Mock()
         self.get_acls_for_lswitches = mock.Mock()
-        self.create_lrouter = mock.Mock()
         self.lrp_del = mock.Mock()
         self.lrp_set_options = mock.Mock()
+        self.lr_add = mock.Mock()
         self.update_lrouter = mock.Mock()
-        self.delete_lrouter = mock.Mock()
+        self.lr_del = mock.Mock()
         self.add_lrouter_port = mock.Mock()
         self.update_lrouter_port = mock.Mock()
         self.delete_lrouter_port = mock.Mock()
@@ -74,7 +76,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.update_acls = mock.Mock()
         self.idl = mock.Mock()
         self.add_static_route = mock.Mock()
-        self.delete_static_route = mock.Mock()
+        self.delete_static_routes = mock.Mock()
         self.get_all_chassis_gateway_bindings = mock.Mock()
         self.get_chassis_gateways = mock.Mock()
         self.get_gateway_chassis_binding = mock.Mock()
@@ -114,11 +116,6 @@ class FakeOvsdbNbOvnIdl(object):
         self.get_floatingip.return_value = None
         self.check_revision_number = mock.Mock()
         self.lookup = mock.MagicMock()
-        # TODO(lucasagomes): The get_floatingip_by_ips() method is part
-        # of a backwards compatibility layer for the Pike -> Queens release,
-        # remove it in the Rocky release.
-        self.get_floatingip_by_ips = mock.Mock()
-        self.get_floatingip_by_ips.return_value = None
         self.get_router_floatingip_lbs = mock.Mock()
         self.get_router_floatingip_lbs.return_value = []
         self.is_col_present = mock.Mock()
@@ -143,6 +140,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.lsp_list = mock.MagicMock()
         self.db_find = mock.Mock()
         self.db_find_rows = mock.Mock()
+        self.db_get = mock.Mock()
         self.db_set = mock.Mock()
         self.db_clear = mock.Mock()
         self.db_remove = mock.Mock()
@@ -163,8 +161,14 @@ class FakeOvsdbNbOvnIdl(object):
         self.ha_chassis_group_del = mock.Mock()
         self.ha_chassis_group_add_chassis = mock.Mock()
         self.ha_chassis_group_del_chassis = mock.Mock()
+        self.get_lrouter_port = mock.Mock()
+        self.get_lrouter_gw_ports = mock.Mock()
         self.lrp_get = mock.Mock()
         self.get_schema_version = mock.Mock(return_value='3.6.0')
+        self.get_lrouter_port = mock.Mock()
+        self.schedule_unhosted_gateways = mock.Mock()
+        self.get_lrouter_by_lrouter_port = mock.Mock()
+        self.schedule_new_gateway = mock.Mock()
 
 
 class FakeOvsdbSbOvnIdl(object):
@@ -180,9 +184,11 @@ class FakeOvsdbSbOvnIdl(object):
         self._get_chassis_physnets = mock.Mock()
         self._get_chassis_physnets.return_value = ['fake-physnet']
         self.get_chassis_and_physnets = mock.Mock()
-        self.get_gateway_chassis_from_cms_options = mock.Mock()
+        self.get_gateway_chassis_from_cms_options = mock.Mock(return_value=[])
+        self.get_extport_chassis_from_cms_options = mock.Mock(return_value=[])
         self.is_col_present = mock.Mock()
         self.is_col_present.return_value = False
+        self.db_find = mock.MagicMock()
         self.db_set = mock.Mock()
         self.lookup = mock.MagicMock()
         self.chassis_list = mock.MagicMock()
@@ -190,6 +196,7 @@ class FakeOvsdbSbOvnIdl(object):
         self.is_table_present.return_value = False
         self.get_chassis_by_card_serial_from_cms_options = mock.Mock()
         self.get_schema_version = mock.Mock(return_value='3.6.0')
+        self.get_chassis_host_for_port = mock.Mock(return_value=set())
 
 
 class FakeOvsdbTransaction(object):
@@ -324,6 +331,7 @@ class FakeNetwork(object):
             'availability_zone_hints': [],
             'is_default': False,
             'standard_attr_id': 1,
+            'mtu': 1500,
         }
 
         # Overwrite default attributes.
@@ -339,6 +347,10 @@ class FakeNetworkContext(object):
         self.fake_segments = segments
         self._plugin_context = mock.MagicMock()
 
+    @property
+    def plugin_context(self):
+        return self._plugin_context
+
     @property
     def current(self):
         return self.fake_network
@@ -359,6 +371,10 @@ class FakeSubnetContext(object):
         self.fake_network = FakeNetworkContext(network, None)
         self._plugin_context = mock.MagicMock()
 
+    @property
+    def plugin_context(self):
+        return self._plugin_context
+
     @property
     def current(self):
         return self.fake_subnet
@@ -404,6 +420,7 @@ class FakeOvsdbRow(FakeResource):
             'delvalue': None,
             'verify': None,
             'setkey': None,
+            'delkey': None,
         }
 
         # Overwrite default attributes and methods.
@@ -486,7 +503,7 @@ class FakePort(object):
         fake_uuid = uuidutils.generate_uuid()
         port_attrs = {
             'admin_state_up': True,
-            'allowed_address_pairs': [{}],
+            'allowed_address_pairs': [],
             'binding:host_id': 'binding-host-id-' + fake_uuid,
             'binding:profile': {},
             'binding:vif_details': {},
@@ -697,6 +714,7 @@ class FakeFloatingIp(object):
 
         # Set default attributes.
         fake_uuid = uuidutils.generate_uuid()
+        standard_attr = FakeStandardAttribute()
         fip_attrs = {
             'id': 'fip-id-' + fake_uuid,
             'tenant_id': '',
@@ -713,7 +731,8 @@ class FakeFloatingIp(object):
             'dns_domain': '',
             'dns_name': '',
             'project_id': '',
-            'standard_attr': FakeStandardAttribute(),
+            'standard_attr': standard_attr,
+            'standard_attr_id': standard_attr.id,
             'qos_policy_binding': FakeQosFIPPolicyBinding(),
             'qos_network_policy_binding': FakeQosNetworkPolicyBinding(),
         }
@@ -725,6 +744,43 @@ class FakeFloatingIp(object):
                             loaded=True)
 
 
+class FakeRouter(object):
+    """Fake one or more Neutron routers."""
+
+    @staticmethod
+    def create_one_router(attrs=None):
+        """Create a fake router.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object faking the router
+        """
+        attrs = attrs or {}
+
+        # Set default attributes.
+        router_id = uuidutils.generate_uuid()
+        router_attrs = {
+            'id': 'router-' + router_id,
+            'name': 'router-' + router_id,
+            'tenant_id': '',
+            'project_id': '',
+            'admin_state_up': True,
+            'status': 'ACTIVE',
+            'gw_port_id': {},
+            'enable_snat': True,
+            'flavor_id': None,
+            'extra_attributes': {},
+            'qos_policy_id': None,
+            'availability_zones': [],
+            'availability_zone_hints': [],
+        }
+
+        # Overwrite default attributes.
+        router_attrs.update(attrs)
+        return FakeResource(info=copy.deepcopy(router_attrs), loaded=True)
+
+
 class FakeOVNPort(object):
     """Fake one or more ports."""
 
@@ -759,7 +815,7 @@ class FakeOVNPort(object):
 
         # Overwrite default attributes.
         port_attrs.update(attrs)
-        return type('Logical_Switch_Port', (object, ), port_attrs)
+        return type('Logical_Switch_Port', (object, ), port_attrs)()
 
     @staticmethod
     def from_neutron_port(port):
@@ -782,7 +838,44 @@ class FakeOVNPort(object):
 
 
 FakeStaticRoute = collections.namedtuple(
-    'Static_Routes', ['ip_prefix', 'nexthop', 'external_ids'])
+    'Static_Routes', ['ip_prefix', 'nexthop', 'external_ids', 'bfd'],
+    defaults=([],))
+
+
+class FakeOVNRouterPort(object):
+    """Fake one or more Logical Router Ports."""
+
+    @staticmethod
+    def create_one_port(attrs=None):
+        """Create a fake ovn Logical Router Port.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object faking the port
+        """
+        attrs = attrs or {}
+
+        # Set default attributes.
+        fake_uuid = uuidutils.generate_uuid()
+        port_attrs = {
+            'enabled': True,
+            'external_ids': {},
+            'gateway_chassis': [],
+            'ha_chassis_group': [],
+            'ipv6_prefix': [],
+            'ipv6_ra_configs': {},
+            'mac': '',
+            'name': fake_uuid,
+            'networks': [],
+            'options': {},
+            'peer': '',
+            'status': '',
+        }
+
+        # Overwrite default attributes.
+        port_attrs.update(attrs)
+        return type('Logical_Router_Port', (object, ), port_attrs)()
 
 
 class FakeOVNRouter(object):
@@ -802,7 +895,7 @@ class FakeOVNRouter(object):
 
         # Overwrite default attributes.
         router_attrs.update(attrs)
-        return type('Logical_Router', (object, ), router_attrs)
+        return type('Logical_Router', (object, ), router_attrs)()
 
     @staticmethod
     def from_neutron_router(router):
@@ -820,6 +913,7 @@ class FakeOVNRouter(object):
                 router.get('name', 'no_router_name')}
 
         # Get the routes
+        ports = []
         routes = []
         for r in router.get('routes', []):
             routes.append(FakeStaticRoute(ip_prefix=r['destination'],
@@ -834,11 +928,16 @@ class FakeOVNRouter(object):
             routes.append(FakeStaticRoute(
                 ip_prefix='0.0.0.0/0', nexthop='',
                 external_ids=external_ids))
+            ports.append(FakeOVNRouterPort.create_one_port(
+                attrs={
+                    'external_ids': {ovn_const.OVN_ROUTER_IS_EXT_GW: 'True'},
+                }))
 
         return FakeOVNRouter.create_one_router(
             {'external_ids': external_ids,
              'enabled': router.get('admin_state_up') or False,
              'name': ovn_utils.ovn_name(router['id']),
+             'ports': ports,
              'static_routes': routes})
 
 
@@ -848,7 +947,8 @@ class FakeChassis(object):
     def create(attrs=None, az_list=None, chassis_as_gw=False,
                bridge_mappings=None, rp_bandwidths=None,
                rp_inventory_defaults=None, rp_hypervisors=None,
-               card_serial_number=None):
+               card_serial_number=None, chassis_as_extport=False,
+               datapath_type=None):
         cms_opts = []
         if az_list:
             cms_opts.append("%s=%s" % (ovn_const.CMS_OPT_AVAILABILITY_ZONES,
@@ -856,6 +956,9 @@ class FakeChassis(object):
         if chassis_as_gw:
             cms_opts.append(ovn_const.CMS_OPT_CHASSIS_AS_GW)
 
+        if chassis_as_extport:
+            cms_opts.append(ovn_const.CMS_OPT_CHASSIS_AS_EXTPORT_HOST)
+
         if rp_bandwidths:
             cms_opts.append('%s=%s' % (n_const.RP_BANDWIDTHS,
                                        ';'.join(rp_bandwidths)))
@@ -871,10 +974,10 @@ class FakeChassis(object):
             cms_opts.append('%s=' % n_const.RP_INVENTORY_DEFAULTS)
 
         if rp_hypervisors:
-            cms_opts.append('%s=%s' % (ovn_const.RP_HYPERVISORS,
+            cms_opts.append('%s=%s' % (n_const.RP_HYPERVISORS,
                                        ';'.join(rp_hypervisors)))
         elif rp_hypervisors == '':  # Test wrongly defined parameter
-            cms_opts.append('%s=' % ovn_const.RP_HYPERVISORS)
+            cms_opts.append('%s=' % n_const.RP_HYPERVISORS)
 
         if card_serial_number:
             cms_opts.append('%s=%s' % (ovn_const.CMS_OPT_CARD_SERIAL_NUMBER,
@@ -890,6 +993,9 @@ class FakeChassis(object):
         if bridge_mappings:
             other_config['ovn-bridge-mappings'] = ','.join(bridge_mappings)
 
+        if datapath_type:
+            other_config[ovn_const.OVN_DATAPATH_TYPE] = datapath_type
+
         chassis_attrs = {
             'encaps': [],
             'external_ids': '',
@@ -902,4 +1008,4 @@ class FakeChassis(object):
 
         # Overwrite default attributes.
         chassis_attrs.update(attrs or {})
-        return type('Chassis', (object, ), chassis_attrs)
+        return type('Chassis', (object, ), chassis_attrs)()
