@@ -28,6 +28,7 @@ from neutron.services.ovn_l3 import plugin as ovn_plugin
 from neutron.tests.unit import fake_resources as fakes
 from neutron.tests.unit.plugins.ml2.drivers.ovn.mech_driver import \
     test_mech_driver
+from neutron.tests.unit.services.logapi.drivers.ovn import test_driver
 
 
 OvnPortInfo = collections.namedtuple('OvnPortInfo', ['name'])
@@ -41,6 +42,7 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
     def setUp(self):
         super(TestOvnNbSyncML2, self).setUp()
 
+        self.test_log_driver = test_driver.TestOVNDriver()
         self.subnet = {'cidr': '10.0.0.0/24',
                        'id': 'subnet1',
                        'subnetpool_id': None,
@@ -428,7 +430,7 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
         mock.patch.object(acl, 'acl_remote_group_id',
                           side_effect=self.matches).start()
         if test_logging:
-            log_objs = [self._fake_log_obj(
+            log_objs = [self.test_log_driver._fake_log_obj(
                 event=log_const.DROP_EVENT, resource_id=None, id='1111')]
             mock.patch.object(ovn_nb_synchronizer.ovn_log_driver, '_get_logs',
                               return_value=log_objs).start()
@@ -502,7 +504,7 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
         ovn_api.delete_lrouter = mock.Mock()
         ovn_api.delete_lrouter_port = mock.Mock()
         ovn_api.add_static_route = mock.Mock()
-        ovn_api.delete_static_route = mock.Mock()
+        ovn_api.delete_static_routes = mock.Mock()
         ovn_api.get_all_dhcp_options.return_value = {
             'subnets': {'n1-s1': {'cidr': '10.0.0.0/24',
                                   'options':
@@ -661,13 +663,14 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
                                                   any_order=True)
         self.assertEqual(len(add_static_route_list),
                          ovn_api.add_static_route.call_count)
-        del_route_calls = [mock.call(mock.ANY, ip_prefix=route['destination'],
-                                     nexthop=route['nexthop'])
+        routes_to_delete = [(route['destination'], route['nexthop'])
                            for route in del_static_route_list]
-        ovn_api.delete_static_route.assert_has_calls(del_route_calls,
-                                                     any_order=True)
-        self.assertEqual(len(del_static_route_list),
-                         ovn_api.delete_static_route.call_count)
+        del_route_call = [mock.call(mock.ANY, routes_to_delete)] \
+            if routes_to_delete else []
+
+        ovn_api.delete_static_routes.assert_has_calls(del_route_call)
+        self.assertEqual(1 if len(del_static_route_list) else 0,
+                         ovn_api.delete_static_routes.call_count)
 
         add_nat_calls = [mock.call(mock.ANY, **nat) for nat in add_snat_list]
         ovn_api.add_nat_rule_in_lrouter.assert_has_calls(add_nat_calls,
@@ -765,16 +768,6 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
             self.assertEqual(4, ovn_nb_synchronizer.ovn_log_driver.
                              _pgs_from_log_obj.call_count)
 
-    def _fake_log_obj(self, **kwargs):
-        log_obj_defaults_dict = {
-            'uuid': uuidutils.generate_uuid(),
-            'resource_id': None,
-            'target_id': None,
-            'event': log_const.ALL_EVENT,
-        }
-        log_obj_obj_dict = {**log_obj_defaults_dict, **kwargs}
-        return mock.Mock(**log_obj_obj_dict)
-
     def _test_ovn_nb_sync_mode_repair(self, test_logging=False):
 
         create_network_list = [{'net': {'id': 'n2', 'mtu': 1450},
