@@ -18,6 +18,7 @@ from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
+from neutron_lib import constants as nlib_consts
 from neutron_lib import exceptions as n_exc
 from neutron_lib.services.trunk import constants as trunk_consts
 from oslo_config import cfg
@@ -25,9 +26,19 @@ from oslo_config import cfg
 from neutron.common.ovn.constants import OVN_ML2_MECH_DRIVER_NAME
 from neutron.objects.ports import Port
 from neutron.objects.ports import PortBinding
+from neutron.objects import trunk as trunk_objects
+from neutron.services.trunk import drivers
 from neutron.services.trunk.drivers.ovn import trunk_driver
+from neutron.services.trunk import plugin as trunk_plugin
 from neutron.tests import base
 from neutron.tests.unit import fake_resources
+from neutron.tests.unit.plugins.ml2 import test_plugin
+
+
+class FakePayload:
+    def __init__(self, trunk, subports=None):
+        self.states = (trunk,)
+        self.metadata = {'subports': subports or []}
 
 
 class TestTrunkHandler(base.BaseTestCase):
@@ -36,7 +47,6 @@ class TestTrunkHandler(base.BaseTestCase):
         self.context = mock.Mock()
         self.plugin_driver = mock.Mock()
         self.plugin_driver._plugin = mock.Mock()
-        self.plugin_driver._plugin.update_port = mock.Mock()
         self.plugin_driver.nb_ovn = fake_resources.FakeOvsdbNbOvnIdl()
         self.handler = trunk_driver.OVNTrunkHandler(self.plugin_driver)
         self.trunk_1 = mock.Mock()
@@ -87,6 +97,8 @@ class TestTrunkHandler(base.BaseTestCase):
             "neutron.objects.ports.PortBinding.update_object").start()
         self.mock_clear_levels = mock.patch(
             "neutron.objects.ports.PortBindingLevel.delete_objects").start()
+        self.mock_bump_revision = mock.patch(
+            "neutron.db.ovn_revision_numbers_db.bump_revision").start()
 
     def _get_fake_port_obj(self, port_id):
         with mock.patch('uuid.UUID') as mock_uuid:
@@ -106,12 +118,20 @@ class TestTrunkHandler(base.BaseTestCase):
 
     def test_create_trunk(self):
         self.trunk_1.sub_ports = []
-        self.handler.trunk_created(self.trunk_1)
+        self.handler.trunk_created(
+            resources.TRUNK,
+            events.AFTER_CREATE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
         self.plugin_driver.nb_ovn.set_lswitch_port.assert_not_called()
         self.mock_update_pb.assert_not_called()
 
         self.trunk_1.sub_ports = [self.sub_port_1, self.sub_port_2]
-        self.handler.trunk_created(self.trunk_1)
+        self.handler.trunk_created(
+            resources.TRUNK,
+            events.AFTER_CREATE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
 
         calls = [mock.call(), mock.call()]
         self._assert_calls(self.mock_port_update, calls)
@@ -129,7 +149,9 @@ class TestTrunkHandler(base.BaseTestCase):
 
         calls = [mock.call(lport_name=s_port.port_id,
                            parent_name=trunk.port_id,
-                           tag=s_port.segmentation_id)
+                           tag=s_port.segmentation_id,
+                           external_ids_update={
+                               'neutron:device_owner': 'trunk:subport'})
                  for trunk, s_port in [(self.trunk_1, self.sub_port_1),
                                        (self.trunk_1, self.sub_port_2)]]
         self._assert_calls(self.plugin_driver.nb_ovn.set_lswitch_port, calls)
@@ -137,14 +159,22 @@ class TestTrunkHandler(base.BaseTestCase):
 
     def test_create_trunk_port_not_found(self):
         self.trunk_1.sub_ports = [self.sub_port_4]
-        self.handler.trunk_created(self.trunk_1)
+        self.handler.trunk_created(
+            resources.TRUNK,
+            events.AFTER_CREATE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
         self.plugin_driver.nb_ovn.set_lswitch_port.assert_not_called()
         self.mock_update_pb.assert_not_called()
 
     def test_create_trunk_port_db_exception(self):
         self.trunk_1.sub_ports = [self.sub_port_1]
         self.mock_update_pb.side_effect = [n_exc.ObjectNotFound(id=1)]
-        self.handler.trunk_created(self.trunk_1)
+        self.handler.trunk_created(
+            resources.TRUNK,
+            events.AFTER_CREATE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
         self.mock_update_pb.assert_called_once_with(
             mock.ANY, {'profile': {'parent_name': self.sub_port_1.trunk_id,
                                    'tag': self.sub_port_1.segmentation_id},
@@ -155,7 +185,11 @@ class TestTrunkHandler(base.BaseTestCase):
 
     def test_delete_trunk(self):
         self.trunk_1.sub_ports = []
-        self.handler.trunk_deleted(self.trunk_1)
+        self.handler.trunk_deleted(
+            resources.TRUNK,
+            events.AFTER_DELETE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
         self.plugin_driver.nb_ovn.set_lswitch_port.assert_not_called()
         self.mock_update_pb.assert_not_called()
         self.mock_clear_levels.assert_not_called()
@@ -169,7 +203,11 @@ class TestTrunkHandler(base.BaseTestCase):
             'tag': self.sub_port_2.segmentation_id,
             'parent_name': self.sub_port_2.trunk_id,
             'foo_field': self.sub_port_2.trunk_id})
-        self.handler.trunk_deleted(self.trunk_1)
+        self.handler.trunk_deleted(
+            resources.TRUNK,
+            events.AFTER_DELETE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
 
         calls = [mock.call(), mock.call()]
         self._assert_calls(self.mock_port_update, calls)
@@ -196,7 +234,8 @@ class TestTrunkHandler(base.BaseTestCase):
         calls = [mock.call(lport_name=s_port.port_id,
                            parent_name=[],
                            tag=[],
-                           up=False)
+                           up=False,
+                           external_ids_update={'neutron:device_owner': ''})
                  for trunk, s_port in [(self.trunk_1, self.sub_port_1),
                                        (self.trunk_1, self.sub_port_2)]]
         self._assert_calls(self.plugin_driver.nb_ovn.set_lswitch_port, calls)
@@ -205,7 +244,11 @@ class TestTrunkHandler(base.BaseTestCase):
         self.sub_port_1_obj.bindings[0].profile.update({
             'foo_field': self.sub_port_1.trunk_id})
         self.trunk_1.sub_ports = [self.sub_port_1]
-        self.handler.trunk_deleted(self.trunk_1)
+        self.handler.trunk_deleted(
+            resources.TRUNK,
+            events.AFTER_DELETE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
         calls = [
             mock.call(mock.ANY,
                       {'profile': {'foo_field': s_port.trunk_id},
@@ -225,13 +268,18 @@ class TestTrunkHandler(base.BaseTestCase):
         calls = [mock.call(lport_name=s_port.port_id,
                            parent_name=[],
                            tag=[],
-                           up=False)
+                           up=False,
+                           external_ids_update={'neutron:device_owner': ''})
                  for trunk, s_port in [(self.trunk_1, self.sub_port_1)]]
         self._assert_calls(self.plugin_driver.nb_ovn.set_lswitch_port, calls)
 
     def test_delete_trunk_port_not_found(self):
         self.trunk_1.sub_ports = [self.sub_port_4]
-        self.handler.trunk_deleted(self.trunk_1)
+        self.handler.trunk_deleted(
+            resources.TRUNK,
+            events.AFTER_DELETE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
         self.plugin_driver.nb_ovn.set_lswitch_port.assert_not_called()
         self.mock_update_pb.assert_not_called()
         self.mock_clear_levels.assert_not_called()
@@ -239,7 +287,11 @@ class TestTrunkHandler(base.BaseTestCase):
     def test_delete_trunk_port_db_exception(self):
         self.trunk_1.sub_ports = [self.sub_port_1]
         self.mock_update_pb.side_effect = [n_exc.ObjectNotFound(id=1)]
-        self.handler.trunk_deleted(self.trunk_1)
+        self.handler.trunk_deleted(
+            resources.TRUNK,
+            events.AFTER_DELETE,
+            self.plugin_driver,
+            FakePayload(self.trunk_1))
         self.mock_update_pb.assert_called_once_with(
             mock.ANY, {'profile': {},
                        'vif_type': portbindings.VIF_TYPE_UNBOUND},
@@ -248,25 +300,8 @@ class TestTrunkHandler(base.BaseTestCase):
         self.plugin_driver.nb_ovn.set_lswitch_port.assert_not_called()
         self.mock_clear_levels.assert_not_called()
 
-    def test_subports_added(self):
-        with mock.patch.object(self.handler, '_set_sub_ports') as set_s:
-            self.handler.subports_added(self.trunk_1,
-                                        [self.sub_port_1, self.sub_port_2])
-        set_s.assert_called_once_with(
-            self.trunk_1.port_id, [self.sub_port_1, self.sub_port_2])
-        self.trunk_1.update.assert_called_once_with(
-            status=trunk_consts.TRUNK_ACTIVE_STATUS)
-
-    def test_subports_deleted(self):
-        with mock.patch.object(self.handler, '_unset_sub_ports') as unset_s:
-            self.handler.subports_deleted(self.trunk_1,
-                                          [self.sub_port_1, self.sub_port_2])
-        unset_s.assert_called_once_with(
-            [self.sub_port_1, self.sub_port_2])
-        self.trunk_1.update.assert_called_once_with(
-            status=trunk_consts.TRUNK_ACTIVE_STATUS)
-
-    def _fake_trunk_event_payload(self):
+    @staticmethod
+    def _fake_trunk_event_payload():
         original_trunk = mock.Mock()
         original_trunk.port_id = 'original_trunk_port_id'
         current_trunk = mock.Mock()
@@ -289,10 +324,13 @@ class TestTrunkHandler(base.BaseTestCase):
         return payload
 
     @mock.patch.object(trunk_driver.OVNTrunkHandler, '_set_sub_ports')
-    def test_trunk_event_create(self, set_subports):
+    def test_trunk_created(self, set_subports):
         fake_payload = self._fake_trunk_event_payload()
-        self.handler.trunk_event(
-            mock.ANY, events.AFTER_CREATE, mock.ANY, fake_payload)
+        self.handler.trunk_created(
+            resources.TRUNK,
+            events.AFTER_CREATE,
+            self.plugin_driver,
+            fake_payload)
         set_subports.assert_called_once_with(
             fake_payload.states[0].port_id,
             fake_payload.states[0].sub_ports)
@@ -300,23 +338,18 @@ class TestTrunkHandler(base.BaseTestCase):
             status=trunk_consts.TRUNK_ACTIVE_STATUS)
 
     @mock.patch.object(trunk_driver.OVNTrunkHandler, '_unset_sub_ports')
-    def test_trunk_event_delete(self, unset_subports):
+    def test_trunk_deleted(self, unset_subports):
         fake_payload = self._fake_trunk_event_payload()
-        self.handler.trunk_event(
-            mock.ANY, events.AFTER_DELETE, mock.ANY, fake_payload)
+        self.handler.trunk_deleted(
+            resources.TRUNK,
+            events.AFTER_DELETE,
+            self.plugin_driver,
+            fake_payload)
         unset_subports.assert_called_once_with(
             fake_payload.states[0].sub_ports)
 
-    @mock.patch.object(trunk_driver.OVNTrunkHandler, '_set_sub_ports')
-    @mock.patch.object(trunk_driver.OVNTrunkHandler, '_unset_sub_ports')
-    def test_trunk_event_invalid(self, unset_subports, set_subports):
-        fake_payload = self._fake_trunk_event_payload()
-        self.handler.trunk_event(
-            mock.ANY, events.BEFORE_DELETE, mock.ANY, fake_payload)
-        set_subports.assert_not_called()
-        unset_subports.assert_not_called()
-
-    def _fake_subport_event_payload(self):
+    @staticmethod
+    def _fake_subport_event_payload():
         original_trunk = mock.Mock()
         original_trunk.port_id = 'original_trunk_port_id'
 
@@ -331,30 +364,139 @@ class TestTrunkHandler(base.BaseTestCase):
 
         return payload
 
-    @mock.patch.object(trunk_driver.OVNTrunkHandler, 'subports_added')
-    def test_subport_event_create(self, s_added):
-        fake_payload = self._fake_subport_event_payload()
-        self.handler.subport_event(
-            mock.ANY, events.AFTER_CREATE, mock.ANY, fake_payload)
-        s_added.assert_called_once_with(
-            fake_payload.states[0], fake_payload.metadata['subports'])
+    def test_subports_added(self):
+        with mock.patch.object(self.handler, '_set_sub_ports') as set_s:
+            self.handler.subports_added(
+                resources.SUBPORTS,
+                events.AFTER_CREATE,
+                self.plugin_driver,
+                FakePayload(self.trunk_1,
+                            [self.sub_port_1, self.sub_port_2]))
+        set_s.assert_called_once_with(
+            self.trunk_1.port_id, [self.sub_port_1, self.sub_port_2])
+        self.trunk_1.update.assert_called_once_with(
+            status=trunk_consts.TRUNK_ACTIVE_STATUS)
 
-    @mock.patch.object(trunk_driver.OVNTrunkHandler, 'subports_deleted')
-    def test_subport_event_delete(self, s_deleted):
+    @mock.patch.object(trunk_driver.OVNTrunkHandler, '_set_sub_ports')
+    def test_subports_added_no_subports(self, m__set_sub_ports):
         fake_payload = self._fake_subport_event_payload()
-        self.handler.subport_event(
-            mock.ANY, events.AFTER_DELETE, mock.ANY, fake_payload)
-        s_deleted.assert_called_once_with(
-            fake_payload.states[0], fake_payload.metadata['subports'])
+        fake_payload.metadata['subports'] = []
+        self.handler.subports_added(
+            resources.SUBPORTS,
+            events.AFTER_CREATE,
+            self.plugin_driver,
+            fake_payload)
+        m__set_sub_ports.assert_not_called()
 
-    @mock.patch.object(trunk_driver.OVNTrunkHandler, 'subports_added')
-    @mock.patch.object(trunk_driver.OVNTrunkHandler, 'subports_deleted')
-    def test_subport_event_invalid(self, s_deleted, s_added):
-        fake_payload = self._fake_trunk_event_payload()
-        self.handler.subport_event(
-            mock.ANY, events.BEFORE_DELETE, mock.ANY, fake_payload)
-        s_added.assert_not_called()
-        s_deleted.assert_not_called()
+    @mock.patch.object(trunk_driver.OVNTrunkHandler, '_set_sub_ports')
+    def test_subports_added_no_parent(self, m__set_sub_ports):
+        fake_payload = self._fake_subport_event_payload()
+        self.plugin_driver.nb_ovn.lookup.return_value = None
+        self.handler.subports_added(
+            resources.SUBPORTS,
+            events.AFTER_CREATE,
+            self.plugin_driver,
+            fake_payload)
+        m__set_sub_ports.assert_not_called()
+
+    def test_subports_deleted(self):
+        with mock.patch.object(self.handler, '_unset_sub_ports') as unset_s:
+            self.handler.subports_deleted(
+                resources.SUBPORTS,
+                events.AFTER_DELETE,
+                self.plugin_driver,
+                FakePayload(self.trunk_1,
+                            [self.sub_port_1, self.sub_port_2]))
+        unset_s.assert_called_once_with(
+            [self.sub_port_1, self.sub_port_2])
+        self.trunk_1.update.assert_called_once_with(
+            status=trunk_consts.TRUNK_ACTIVE_STATUS)
+
+    @mock.patch.object(trunk_driver.OVNTrunkHandler, '_unset_sub_ports')
+    def test_subports_deleted_no_subports(self, m__unset_sub_ports):
+        fake_payload = self._fake_subport_event_payload()
+        fake_payload.metadata['subports'] = []
+        self.handler.subports_deleted(
+            resources.SUBPORTS,
+            events.AFTER_DELETE,
+            self.plugin_driver,
+            fake_payload)
+        m__unset_sub_ports.assert_not_called()
+
+    @mock.patch.object(trunk_driver.OVNTrunkHandler, '_unset_sub_ports')
+    def test_subports_deleted_no_parent(self, m__unset_sub_ports):
+        fake_payload = self._fake_subport_event_payload()
+        self.plugin_driver.nb_ovn.lookup.return_value = None
+        self.handler.subports_deleted(
+            resources.SUBPORTS,
+            events.AFTER_DELETE,
+            self.plugin_driver,
+            fake_payload)
+        m__unset_sub_ports.assert_not_called()
+
+
+class TestTrunkHandlerWithPlugin(test_plugin.Ml2PluginV2TestCase):
+    def setUp(self):
+        super().setUp()
+        self.drivers_patch = mock.patch.object(drivers, 'register').start()
+        self.compat_patch = mock.patch.object(
+            trunk_plugin.TrunkPlugin, 'check_compatibility').start()
+        self.trunk_plugin = trunk_plugin.TrunkPlugin()
+        self.trunk_plugin.add_segmentation_type('vlan', lambda x: True)
+        self.plugin_driver = mock.Mock()
+        self.trunk_handler = trunk_driver.OVNTrunkHandler(self.plugin_driver)
+
+    def _create_test_trunk(self, port, subports=None):
+        subports = subports if subports else []
+        trunk = {'port_id': port['port']['id'],
+                 'project_id': 'test_tenant',
+                 'sub_ports': subports}
+        response = (
+            self.trunk_plugin.create_trunk(self.context, {'trunk': trunk}))
+        return response
+
+    def _get_trunk_obj(self, trunk_id):
+        return trunk_objects.Trunk.get_object(self.context, id=trunk_id)
+
+    def test_parent_active_triggers_trunk_active(self):
+        with self.port() as new_parent:
+            new_parent['status'] = nlib_consts.PORT_STATUS_ACTIVE
+            old_parent = {'status': nlib_consts.PORT_STATUS_DOWN}
+            old_trunk = self._create_test_trunk(new_parent)
+            old_trunk = self._get_trunk_obj(old_trunk['id'])
+            old_trunk.update(status=trunk_consts.TRUNK_DOWN_STATUS)
+            trunk_details = {'trunk_id': old_trunk.id}
+            new_parent['trunk_details'] = trunk_details
+            old_parent['trunk_details'] = trunk_details
+            self.trunk_handler.port_updated(
+                resources.PORT,
+                events.AFTER_UPDATE,
+                None,
+                payload=events.DBEventPayload(
+                    self.context, states=(old_parent, new_parent)))
+            new_trunk = self._get_trunk_obj(old_trunk.id)
+            self.assertEqual(
+                trunk_consts.TRUNK_ACTIVE_STATUS, new_trunk.status)
+
+    def test_parent_build_does_not_trigger_trunk_active(self):
+        with self.port() as new_parent:
+            new_parent['status'] = nlib_consts.PORT_STATUS_BUILD
+            old_parent = {'status': nlib_consts.PORT_STATUS_DOWN}
+            old_trunk = self._create_test_trunk(new_parent)
+            old_trunk = self._get_trunk_obj(old_trunk['id'])
+            old_trunk.update(status=trunk_consts.TRUNK_DOWN_STATUS)
+            trunk_details = {'trunk_id': old_trunk.id}
+            new_parent['trunk_details'] = trunk_details
+            old_parent['trunk_details'] = trunk_details
+            self.trunk_handler.port_updated(
+                resources.PORT,
+                events.AFTER_UPDATE,
+                None,
+                payload=events.DBEventPayload(
+                    self.context, states=(old_parent, new_parent)))
+            new_trunk = self._get_trunk_obj(old_trunk.id)
+            self.assertNotEqual(
+                trunk_consts.TRUNK_ACTIVE_STATUS, new_trunk.status)
 
 
 class TestTrunkDriver(base.BaseTestCase):
@@ -380,16 +522,14 @@ class TestTrunkDriver(base.BaseTestCase):
         driver = trunk_driver.OVNTrunkDriver.create(mock.Mock())
         with mock.patch.object(registry, 'subscribe') as mock_subscribe:
             driver.register(mock.ANY, mock.ANY, mock.Mock())
-            calls = [mock.call.mock_subscribe(mock.ANY,
-                                              resources.TRUNK,
-                                              events.AFTER_CREATE),
-                     mock.call.mock_subscribe(mock.ANY,
-                                              resources.SUBPORTS,
-                                              events.AFTER_CREATE),
-                     mock.call.mock_subscribe(mock.ANY,
-                                              resources.TRUNK,
-                                              events.AFTER_DELETE),
-                     mock.call.mock_subscribe(mock.ANY,
-                                              resources.SUBPORTS,
-                                              events.AFTER_DELETE)]
+            calls = [
+                mock.call.mock_subscribe(
+                    mock.ANY, resources.TRUNK, events.AFTER_CREATE),
+                mock.call.mock_subscribe(
+                    mock.ANY, resources.TRUNK, events.AFTER_DELETE),
+                mock.call.mock_subscribe(
+                    mock.ANY, resources.SUBPORTS, events.AFTER_CREATE),
+                mock.call.mock_subscribe(
+                    mock.ANY, resources.SUBPORTS, events.AFTER_DELETE),
+            ]
             mock_subscribe.assert_has_calls(calls, any_order=True)
