@@ -26,7 +26,6 @@ from neutron_lib import exceptions
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from oslo_config import cfg
-from oslo_db import exception as db_exc
 from oslo_policy import fixture as op_fixture
 from oslo_policy import policy as oslo_policy
 from oslo_serialization import jsonutils
@@ -110,10 +109,10 @@ class PolicyTestCase(base.BaseTestCase):
         cfg.CONF.set_override(
             'enforce_new_defaults', enforce_new_defaults, group='oslo_policy')
         project_admin_ctx = context.Context(
-            user="fake", project_id="fake",
+            user_id="fake", project_id="fake",
             roles=['admin', 'member', 'reader'])
         system_admin_ctx = context.Context(
-            user="fake",
+            user_id="fake",
             roles=['admin', 'member', 'reader'],
             system_scope='all')
         if not enforce_new_defaults:
@@ -138,10 +137,10 @@ class PolicyTestCase(base.BaseTestCase):
         cfg.CONF.set_override(
             'enforce_new_defaults', enforce_new_defaults, group='oslo_policy')
         project_admin_ctx = context.Context(
-            user="fake", project_id="fake",
+            user_id="fake", project_id="fake",
             roles=['admin', 'member', 'reader'])
         system_admin_ctx = context.Context(
-            user="fake",
+            user_id="fake",
             roles=['admin', 'member', 'reader'],
             system_scope='all')
         if not enforce_new_defaults:
@@ -194,7 +193,7 @@ class PolicyTestCase(base.BaseTestCase):
         action = "get_example:only_project_user_allowed"
         target = {'project_id': 'some-project'}
         system_admin_ctx = context.Context(
-            user="fake",
+            user_id="fake",
             roles=['admin', 'member', 'reader'],
             system_scope='all')
         self.assertFalse(policy.check(system_admin_ctx, action, target))
@@ -696,14 +695,16 @@ class NeutronPolicyTestCase(base.BaseTestCase):
     def test_retryrequest_on_notfound(self):
         failure = exceptions.NetworkNotFound(net_id='whatever')
         action = "create_port:mac"
-        with mock.patch.object(directory.get_plugin(),
-                               'get_network', side_effect=failure):
+        with mock.patch.object(
+                directory.get_plugin(),
+                'get_network', side_effect=failure) as get_network_mock:
             target = {'network_id': 'whatever'}
             try:
                 policy.enforce(self.context, action, target)
-                self.fail("Did not raise RetryRequest")
-            except db_exc.RetryRequest as e:
-                self.assertEqual(failure, e.inner_exc)
+                self.fail("Did not raise NotFound exception and retry "
+                          "DB request.")
+            except exceptions.NetworkNotFound:
+                self.assertEqual(2, get_network_mock.call_count)
 
     def test_enforce_tenant_id_check_parent_resource_bw_compatibility(self):
 
