@@ -23,7 +23,6 @@ from oslo_config import cfg
 
 from neutron import service
 from neutron.tests import base
-from neutron.tests.unit import test_wsgi
 
 
 class TestServiceHelpers(base.BaseTestCase):
@@ -34,10 +33,19 @@ class TestServiceHelpers(base.BaseTestCase):
         self.assertLessEqual(num_workers, processutils.get_worker_count())
 
 
-class TestRpcWorker(test_wsgi.TestServiceBase):
+class TestRpcWorker(base.BaseTestCase):
+
+    @mock.patch("neutron.policy.refresh")
+    @mock.patch("neutron.common.config.setup_logging")
+    def _test_reset(self, worker_service, setup_logging_mock, refresh_mock):
+        worker_service.reset()
+
+        setup_logging_mock.assert_called_once_with()
+        refresh_mock.assert_called_once_with()
 
     def test_reset(self):
         _plugin = mock.Mock()
+
         rpc_worker = service.RpcWorker(_plugin)
         self._test_reset(rpc_worker)
 
@@ -54,12 +62,15 @@ class TestRunRpcWorkers(base.BaseTestCase):
             with mock.patch('neutron.service.RpcReportsWorker'):
                 service._get_rpc_workers(plugin=mock.Mock())
         init_call = mock_rpc_worker.call_args
-        expected_call = mock.call(
-            mock.ANY, worker_process_count=expected_passed_value)
-        self.assertEqual(expected_call, init_call)
+        if expected_passed_value > 0:
+            expected_call = mock.call(
+                mock.ANY, worker_process_count=expected_passed_value)
+            self.assertEqual(expected_call, init_call)
+        else:
+            mock_rpc_worker.assert_not_called()
 
     def test_rpc_workers_zero(self):
-        self._test_rpc_workers(0, 1)
+        self._test_rpc_workers(0, 0)
 
     def test_rpc_workers_default_api_workers_default(self):
         workers = max(int(self.worker_count / 2), 1)
@@ -81,7 +92,7 @@ class TestRunWsgiApp(base.BaseTestCase):
     def _test_api_workers(self, config_value, expected_passed_value):
         if config_value is not None:
             cfg.CONF.set_override('api_workers', config_value)
-        with mock.patch('neutron.wsgi.Server') as mock_server:
+        with mock.patch('neutron.api.wsgi.Server') as mock_server:
             service.run_wsgi_app(mock.sentinel.app)
         start_call = mock_server.return_value.start.call_args
         expected_call = mock.call(
@@ -89,8 +100,8 @@ class TestRunWsgiApp(base.BaseTestCase):
             workers=expected_passed_value)
         self.assertEqual(expected_call, start_call)
 
-    def test_api_workers_zero(self):
-        self._test_api_workers(0, 0)
+    def test_api_workers_one(self):
+        self._test_api_workers(1, 1)
 
     def test_api_workers_default(self):
         self._test_api_workers(None, self.worker_count)
@@ -99,7 +110,7 @@ class TestRunWsgiApp(base.BaseTestCase):
         self._test_api_workers(42, 42)
 
     def test_start_all_workers(self):
-        cfg.CONF.set_override('api_workers', 0)
+        cfg.CONF.set_override('api_workers', 1)
         mock.patch.object(service, '_get_rpc_workers').start()
         mock.patch.object(service, '_get_plugins_workers').start()
         mock.patch.object(service, '_start_workers').start()
